#ifndef Dabble_h
#define Dabble_h

#include "Stream.h"
#include "Arduino.h"
#include "ModuleIds.h"
#include "ModuleSelection.h"
#include "ModuleIncludes.h"
#include "ModuleParent.h"
#include "ModuleInstantiation.h"
#include "InternetModule.h"
#include "HttpResponse.h"
#include "HttpRequest.h"

#define ONE_SECOND 1000

//Start and End of packet sent
#define START_OF_FRAME  0xFF
#define END_OF_FRAME 	0x00
//Library Version
#define LIBRARY_VERSION 1
//Time between sending Frames
#define TIME_GAP		10UL

//Selecting picture from folder
#define FROM_DABBLE_FOLDER 0x00
#define FROM_CAMERA_FOLDER	  0x01


//#define DEBUG
//#define DEBUG

//Output function ID's
#define SEND_LIBRARY_VERSION	0x01
#define CHECK_APP_CONNECTION	0x02
#define CALLBACK_ENTERED		0x03
#define CALLBACK_EXITED			0x04
#define LIBRARY_TESTING_RESPONSE	0x05
//Input function ID's 
//Checking Bluetooth connection
#define CONNECTION_CHECK_FUNCTION 0x01
#define DISCONNECTION_CHECK_FUNCTION 0x02


#define BOARDID_REQUEST	0x03
//#define LIBRARY_VERSION_REQUEST	0x03
//#define LIBRARY_TESTING_REQUEST	0x05


//Numer of Module
#define MODULE_NO	11

//Class for Datalength and Data
class FunctionArg
{
private:
	byte length;
	byte * data;
	bool saveData;
public:
	FunctionArg(int l ,byte * d, bool _saveData=false)
	{
		saveData=_saveData;
		length=(l>0xff)?0xff:l;
		if(saveData)
		{
			data=(byte *)malloc(sizeof(byte)*length);
			memcpy(data,d,length);
		}
		else
		{
			data=d;
		}
	}
	byte getLength()
	{
		return length;
	}
	byte * getData()
	{
		return data;
	}
	~FunctionArg()
	{
		if(saveData)free(data);
	}

};

union FloatUnion{
	    float number;
	    byte floatBytes[sizeof(float)];
	  };



class DabbleClass
{


public:
    
	DabbleClass();
	//Blocking function
	void waitForAppConnection();
	//Check connection
	bool isAppConnected();
	void setOnAppConnected(void (*)(bool));
	//rx,tx pins for software
	uint8_t pin_rx,pin_tx;
	//Getters 
	byte getModuleId();
	byte getFunctionId();
	byte getArgumentNo();
	byte getScreenId();
	byte readModuleId();
	byte getArgumentLength(byte );
	byte * getArgumentData(byte );
	void convertFloatToBytes(float , byte *);
	float convertBytesToFloat(byte * );
	//Processing Incomming Frames
	void processInput();		
	//Library Starter
	void begin();
	void begin(Stream &s);
	//void begin(long baudRate);
	void begin(long baudRate=115200,int rx=2,int tx=3);
	//Adding objects in array 
	static void addToModulesArray(ModuleParent *);
	// #ifdef INTERNET_MODULE
	static void addToUnSentRequestsArray(HttpRequest *);
	// #endif
	static bool isInitialized();
	static bool isSoftwareSerial();
	//Frame Sender
	void sendModuleFrame(byte , byte ,byte , byte , ...);
	void sendModuleFrame(byte , byte , byte , byte , FunctionArg ** );
	void setOnNewModuleFrame(void (*)(byte, byte, byte, byte *,byte **));
	void setOnNewSerialData(void (*)(byte));
	//PulseWidthModulation Getter 
	unsigned char analogRead(int );	
	void delay(unsigned long);
	bool isCallbacksInterruptsSet();
	void enableCallbacksInterrupts();
	void disableCallbacksInterrupts();
	byte getVerificationByte();
    static Stream * DabbleSerial;
private:
	//Reserve Variables
	bool isArgumentsNumberMalloced;
	bool isArgumentLengthMalloced;
	bool isDabbleConnected;
	bool isAppConnectionCallBack;
	bool isModuleFrameCallback;
	bool isSerialDataCallback;
	bool dontDelay;
	static bool isFirstFrame;
	bool framestart;
	static bool inACallback;
	static bool callbacksInterrupts;
	//Data bytes
	byte numberOfDataMalloced;
	byte Module;
	byte verificationByte;
	byte functions;
	byte counter;
	byte screenId;
	byte readModuleID;
	byte argumentcounter;
	byte datalengthcounter;
	byte argumentnumber;
	byte **arguments;	
	byte *argumentL;            
	byte endFrame;
	//Module Counter 
	static byte ModulesCounter;
	//Requests Counter
	static byte requestsCounter;
	//Is constructor called
	static bool isInit;
	static bool isSws;
	//Checker variable 
	static unsigned long lastTimeFrameSent;

	static unsigned long argumentDataBytesTimeReceived;
	//Array of pointers to Parents
	static ModuleParent * ModulesArray[MODULE_NO];
	// #ifdef INTERNET_MODULE
	//Array of pointers to un sent requests
	static HttpRequest ** requestsArray;
	// #endif
	//Send Incomming Data to Modules
	void sendToModules();
    //void begin(long baudRate);
	void init();
	void freeMemoryAllocated();
	void processFrame();
	void (*isAppConnectedCallBack)(bool);
	void (*ModuleFrameCallback)(byte, byte, byte, byte *,byte **);
	void (*serialDataCallback)(byte);
	void enteringACallback();
	void exitingACallback();
	bool isInACallback();
	void processInput(int);
	void appWrite(byte data);
friend class ModuleParent;
};
//Extern Object
extern DabbleClass Dabble;
#endif